
#include "badge_init.h"
#include "badge_basics.h"
#include "badge_lcd.h"
#include <msp430x14x.h>

#include "badge_lcd_chars.h"

/* -------------------- High-Level LCD Routines ------------------------ */

void LCD_Backlight_Dimmer_Init(unsigned char lcd_num)
{
  TACTL = TASSEL0 + TACLR;              // ACLK, Clear Tar
  CCR0 = 512-1;                         // PWM Period
  CCTL1 = OUTMOD_7;  			// CCR1 reset/set
  CCR1 = 0;                           // CCR1 PWM duty cycle
  CCTL2 = OUTMOD_7;  			// CCR2 reset/set
  CCR2 = 0;                           // CCR2 PWM duty cycle
  if (lcd_num == LCD1){
    P1DIR |= 0x04;
    P1SEL |= 0x04;  
  } else {
    P1DIR |= 0x08;                        // P1.2 and P1.3 output
    P1SEL |= 0x08;                        // P1.2 and P1.3 TA1/2 otions
  }
  TACTL |= MC0;                         // Start Timer_A in up mode
}

void LCD_Set_Backlight_Intensity(unsigned short intensity,unsigned char lcd_num)
{
  if (lcd_num == LCD1){
    CCR1 = intensity;                           // CCR1 PWM duty cycle
  } else {
    CCR2 = intensity;                           // CCR2 PWM duty cycle
  }
}

/*
** LCD_Init: Initialize the LCD.
*/
void LCD_Init (unsigned char lcd_num)
{
  P5DIR = 0xFF;
  P5SEL = 0x00;
  P5OUT = 0x00;
  P1DIR |= BL1+BL2+RS+RW;
  P1OUT |= RS+RW;
  P1OUT &= ~(BL1+BL2);
  P1SEL &= ~(RS+RW+BL1+BL2);
  P2DIR |= E1 + E2;
  P2SEL &= ~(E1 + E2);
  P2OUT &= ~(E1 + E2);


	LCD_InitDriver(lcd_num);
	LCD_Clear(lcd_num);
	LCD_CursorOff(lcd_num);

	//
	// load user-defined characters into LCD
	//
	LCD_DefineChar (0, LCD_CHAR1,lcd_num);
	LCD_DefineChar (1, LCD_CHAR2,lcd_num);
	LCD_DefineChar (2, LCD_CHAR3,lcd_num);
	LCD_DefineChar (3, LCD_CHAR4,lcd_num);
	LCD_DefineChar (4, LCD_CHAR5,lcd_num);
	LCD_DefineChar (5, LCD_CHAR6,lcd_num);
	LCD_DefineChar (6, LCD_CHAR7,lcd_num);
	LCD_DefineChar (7, LCD_CHAR8,lcd_num);
}


/*
** LCD_Clear: Clear the LCD screen (also homes cursor).
*/
void LCD_Clear (unsigned char lcd_num)
{
	LCD_WriteControl(0x01,lcd_num);
}


/*
** LCD_Home: Position the LCD cursor at row 1, col 1.
*/
void LCD_Home (unsigned char lcd_num)
{
	LCD_WriteControl(0x02,lcd_num);
}


/*
** LCD_DisplayCharacter: Display a single character,
**                       at the current cursor location.
*/
void LCD_DisplayCharacter (unsigned char a_char, unsigned char lcd_num)
{
	LCD_WriteData (a_char,lcd_num);
}


/*
** LCD_DisplayString: Display a string at the specified row and column.
*/
void LCD_DisplayString (unsigned char row, unsigned char column, char *string, unsigned char lcd_num)
{
	LCD_Cursor (row, column,lcd_num);
	while (*string)
		LCD_DisplayCharacter (*string++,lcd_num);
}


/*
** LCD_DisplayStringCentered: Display a string centered on the specified row.
*/
void LCD_DisplayStringCentered (unsigned char row, char *string, unsigned char lcd_num)
{
	char len = strlen (string);

	if (len <= 20) {
		// if the string is less than one line, center it ...
		char i;
		LCD_Cursor (row, 1,lcd_num);
		for (i=0; i<20; i++)
			LCD_DisplayCharacter (' ',lcd_num);
		LCD_DisplayString(row,((20 - len) / 2)+1,string,lcd_num);
	}
	else {
		// if the string is more than one line, display first 20 characters
		char temp[21];
		strncpy(temp, string, 20);
		temp[20] = 0;
		LCD_DisplayString(row,1,temp,lcd_num);
	}
}


/*
** LCD_Cursor: Position the LCD cursor at "row", "column".
*/
void LCD_Cursor (unsigned char row, unsigned char column, unsigned char lcd_num)
{
	switch (row) {
		case 1: LCD_WriteControl (0x80 + column - 1,lcd_num); break;
		case 2: LCD_WriteControl (0xc0 + column - 1,lcd_num); break;
		case 3: LCD_WriteControl (0x94 + column - 1,lcd_num); break;
		case 4: LCD_WriteControl (0xd4 + column - 1,lcd_num); break;
		default: break;
	}
}


/*
** LCD_DisplayScreen: Display an entire screen (80 characters).
**
**  inputs: ptr = pointer to a string containing the entire screen
**
**  example:
**		char screen[] = "01234567890123456789"
**										" This is a test of  "
**										"LCD_DisplayScreen()."
**										"   How's it look?   ";
**		DisplayScreen(screen);
**
*/
void LCD_DisplayScreen (unsigned char *ptr, unsigned char lcd_num)
{
	LCD_DisplayRow(1,ptr+ 0,lcd_num);
	LCD_DisplayRow(2,ptr+20,lcd_num);
	LCD_DisplayRow(3,ptr+40,lcd_num);
	LCD_DisplayRow(4,ptr+60,lcd_num);
}


/*
** LCD_WipeOnLR: Display an entire screen (80 characters) by
**               "wiping" it on (left to right).
**
**  inputs: ptr = pointer to a string containing the entire screen
**
*/
void LCD_WipeOnLR (unsigned char *ptr, unsigned char lcd_num)
{
	// "wipe" on new screen
	unsigned char i;
	for (i=0; i<20; i++) {
		LCD_Cursor(1,i+1,lcd_num);
		LCD_DisplayCharacter(*(ptr+ 0+i),lcd_num);
		LCD_Cursor(2,i+1,lcd_num);
		LCD_DisplayCharacter(*(ptr+20+i),lcd_num);
		LCD_Cursor(3,i+1,lcd_num);
		LCD_DisplayCharacter(*(ptr+40+i),lcd_num);
		LCD_Cursor(4,i+1,lcd_num);
		LCD_DisplayCharacter(*(ptr+60+i),lcd_num);
	}
}


/*
** LCD_WipeOnLR: Display an entire screen (80 characters) by
**               "wiping" it on (right to left).
**
**  inputs: ptr = pointer to a string containing the entire screen
**
*/
void LCD_WipeOnRL (unsigned char *ptr, unsigned char lcd_num)
{
	// "wipe" on new screen
	unsigned char i;
	for (i=20; i>0; i--) {
		LCD_Cursor(1,i,lcd_num);
		LCD_DisplayCharacter(*(ptr+ 0+i-1),lcd_num);
		LCD_Cursor(2,i,lcd_num);
		LCD_DisplayCharacter(*(ptr+20+i-1),lcd_num);
		LCD_Cursor(3,i,lcd_num);
		LCD_DisplayCharacter(*(ptr+40+i-1),lcd_num);
		LCD_Cursor(4,i,lcd_num);
		LCD_DisplayCharacter(*(ptr+60+i-1),lcd_num);
	}
}


/*
** LCD_WipeOffLR: "Wipe" screen left-to-right.
*/
void LCD_WipeOffLR (unsigned char lcd_num)
{
	// "wipe" off old screen (left to right)
	unsigned char i;
	for (i=1; i<21; i++) {
		#define BLOCK 0xff
		LCD_Cursor(1,i,lcd_num);
		LCD_DisplayCharacter(BLOCK,lcd_num);
		LCD_Cursor(2,i,lcd_num);
		LCD_DisplayCharacter(BLOCK,lcd_num);
		LCD_Cursor(3,i,lcd_num);
		LCD_DisplayCharacter(BLOCK,lcd_num);
		LCD_Cursor(4,i,lcd_num);
		LCD_DisplayCharacter(BLOCK,lcd_num);
	}
}


/*
** LCD_WipeOffRL: "Wipe" screen right-to-left.
*/
void LCD_WipeOffRL (unsigned char lcd_num)
{
	// "wipe" off old screen (right to left)
	unsigned char i;
	for (i=20; i>0; i--) {
		#define BLOCK 0xff
		LCD_Cursor(1,i,lcd_num);
		LCD_DisplayCharacter(BLOCK,lcd_num);
		LCD_Cursor(2,i,lcd_num);
		LCD_DisplayCharacter(BLOCK,lcd_num);
		LCD_Cursor(3,i,lcd_num);
		LCD_DisplayCharacter(BLOCK,lcd_num);
		LCD_Cursor(4,i,lcd_num);
		LCD_DisplayCharacter(BLOCK,lcd_num);
	}
}



/*
** LCD_DisplayRow: Display a string at the specified row.
*/
void LCD_DisplayRow (unsigned char row, unsigned char *string, unsigned char lcd_num)
{
	unsigned char i;
	LCD_Cursor (row, 1,lcd_num);
	for (i=0; i<20; i++)
		LCD_DisplayCharacter (*string++,lcd_num);
}


/*
** LCD_CursorLeft: Move the cursor left by one character.
*/
void LCD_CursorLeft (unsigned char lcd_num)
{
	LCD_WriteControl (0x10,lcd_num);
}


/*
** LCD_CursorRight: Move the cursor right by one character.
*/
void LCD_CursorRight (unsigned char lcd_num)
{
	LCD_WriteControl (0x14,lcd_num);
}


/*
** LCD_CursorOn: Turn the cursor on.
*/
void LCD_CursorOn (unsigned char lcd_num)
{
	LCD_WriteControl (0x0d,lcd_num);
}


/*
** LCD_CursorOff: Turn the cursor off.
*/
void LCD_CursorOff (unsigned char lcd_num)
{
	LCD_WriteControl (0x0c,lcd_num);
}


/*
** LCD_DisplayOff: Turn Off LCD.
*/
void LCD_DisplayOff (unsigned char lcd_num)
{
	LCD_WriteControl(0x08,lcd_num);
}


/*
** LCD_DisplayOn: Turn On LCD.
*/
void LCD_DisplayOn (unsigned char lcd_num)
{
	LCD_WriteControl(0x0c,lcd_num);
}


/*
** LCD_DefineCharacter: Define dot pattern for user-defined character.
**
**  inputs: address = address of character (0x00-0x07)
**          pattern = pointer to 8-byte array containing the dot pattern
*/
void LCD_DefineChar (unsigned char address, const unsigned char *pattern, unsigned char lcd_num)
{
	unsigned char i;

	LCD_WriteControl(0x40 + (address << 3),lcd_num);
	for (i=0; i<8; i++) {
		LCD_WriteData(*pattern++,lcd_num);
	}
}



/* -------------------- Low-Level LCD Routines ------------------------ */

/*
** LCD_InitDriver: Initialize the LCD driver.
*/
void LCD_InitDriver (unsigned char lcd_num)
{
	LCD_WriteControl(0x38,lcd_num);
	LCD_WriteControl(0x38,lcd_num);
	LCD_WriteControl(0x38,lcd_num);
	LCD_WriteControl(0x06,lcd_num);
	LCD_WriteControl(0x0c,lcd_num);
}


/*
** LCD_WriteControl: Write a control instruction to the LCD
*/
void LCD_WriteControl (unsigned char in_data, unsigned char lcd_num)
{
	P1OUT &= ~(RS);
	P1OUT &= ~(RW);
        if (lcd_num == LCD1){
          P2OUT |= E1;
        } else {
          P2OUT |= E2;
        }
	LCD_DATA = in_data;
        Wait(1);
        if (lcd_num == LCD1){
          P2OUT &= ~E1;
        } else {
          P2OUT &= ~E2;
        }
        Wait(1);
}


/*
** LCD_WriteData: Write one byte of data to the LCD
*/
void LCD_WriteData (unsigned char in_data, unsigned char lcd_num)
{
	P1OUT &= ~(RW);
	P1OUT |= RS;
        if (lcd_num == LCD1){
          P2OUT |= E1;
        } else {
          P2OUT |= E2;
        }
	LCD_DATA = in_data;
        Wait(1);
        if (lcd_num == LCD1){
          P2OUT &= ~E1;
        } else {
          P2OUT &= ~E2;
        }
        Wait(1);
}
